/*
   nice processor: logo prank emulating AMD's font design for Ryzen
*/

#version 3.7;
#global_settings {
  assumed_gamma 1.0
  max_trace_level 10  
}

camera {
  orthographic
  sky       <0, 1, 0>
  up        0.9*<0, 2.4, 0>
  right     0.9*<6.4, 0, 0>
  direction <0, 0, -4>
  location  <0, 0, 10>
  look_at   <0, 0, 0>
}

#default {
  texture {
    finish {
      ambient 1.0
      diffuse 0.0
      specular 0.0
    }
    pigment {color rgb 1}
  }
}


// diagonal Z bar coefficients
#declare dza = (5 + 3*sqrt(33))/34;
#declare dzb = -(5*sqrt(27)*sqrt(11) - 9)/102;
#declare dzc = (5*sqrt(3)*sqrt(11) - 3)/34;

// letter Z
#declare LetterZ = union {  // black parts
  union {
    box {<0,4,0>,<5,5,1>}  // upper horizontal bar
    box {<0,0,0>,<5,1,1>}  // lower horizontal bar
    intersection {
      box {<0,0,0>,<5,5,1>}
      plane {<-dza,-dzb,0>,dzc}
      plane {<-dza,-dzb,0>,dzc
        translate < -2.5, -2.5, 0 >
	rotate 180*z
        translate < 2.5, 2.5, 0 >
      }
    }
  }
  translate < -2.5, -2.5, 0 >
}


// letter N
#declare LetterN = object { LetterZ
  rotate 90*z
}


// letter i
#declare Letteri = union {
  box {<2,0,0>,<3,3.5,1>}  // vertical bar
  //sphere {<2.5,4.5,0.5>,0.5}  // i dot
  box {<2,4,0>,<3,5,1>}  // i dot
  translate < -2.5, -2.5, 0 >
}


// letter C
#declare LetterC = difference {
  union {
    box{<0,1,0>,<5,4,1>}  // left to right
    box{<1,0,0>,<4,5,1>}  // top to bottom
    cylinder {<1,1,0>,<1,1,1>,1}  // bottom left
    cylinder {<4,1,0>,<4,1,1>,1}  // bottom right
    cylinder {<1,4,0>,<1,4,1>,1}  // top left
    cylinder {<4,4,0>,<4,4,1>,1}  // top right
  }
  box {<1,1,-1>,<6,4,2>}
  //box {<1,1.5,-1>,<6,3.5,2>}
  box {<-1,3,-1>,<2,4,2>}
  translate < -2.5, -2.5, 0 >
}


// letter E
#declare LetterE = union {
  box {<0,4,0>,<5,5,1>}  // upper horizontal bar
  box {<0,0,0>,<5,1,1>}  // lower horizontal bar
  //box {<0,2,0>,<4,3,1>}  // middle horizontal bar
  box {<0,2,0>,<5,3,1>}  // middle horizontal bar
  box {<0,0,0>,<1,3,1>} // left vertical bar

  translate < -2.5, -2.5, 0 >
}

// word NiCE
#declare WordNiCE = union {
  object {LetterN
    translate -12*x
  }
  object {Letteri
    translate -8*x
  }
  object {LetterC
    translate -4*x
  }
  object {LetterE
    translate 2*x
  }

  translate 0.5*(3 + 1 + 3 + 3)*x
  //scale 0.25
}

// letter P
#declare LetterP = union {
  union {  // black parts
    box {<0,4,0>,<4,5,0>}  // upper horizontal bar
    box {<0,2,0>,<4,3,0>}  // lower horizontal bar
    box {<0,0,0>,<1,3,1>}  // left vertical bar
    box {<4,3,0>,<5,4,1>}  // right vertical bar
  }
  difference {
    union {
      cylinder {<4,4,0>,<4,4,1>,1}
      cylinder {<4,3,0>,<4,3,1>,1}
    }
    box {<0,3,-1>,<4,4,2>}
  }
  translate < -2.5, -2.5, 0 >
}

#declare LetterR = union {
  object {LetterP
    translate < 2.5, 2.5, 0 >
  }

  intersection {
    plane { < 2, 1, 0 >, 0
      translate 5*x
    }
    plane { < -2, -1, 0 >, 0
      translate 5*x
      translate 1/sqrt(5)*<-2, -1, 0>
    }
    box {<1,0,0>,<5,2,1>}
  }

  translate < -2.5, -2.5, 0 >
}

#declare LetterO = difference {
  union {
    box{<0,1,0>,<5,4,1>}  // left to right
    box{<1,0,0>,<4,5,1>}  // top to bottom
    cylinder {<1,1,0>,<1,1,1>,1}  // bottom left
    cylinder {<4,1,0>,<4,1,1>,1}  // bottom right
    cylinder {<1,4,0>,<1,4,1>,1}  // top left
    cylinder {<4,4,0>,<4,4,1>,1}  // top right
  }
  box {<1,1,-1>,<4,4,2>}  // inner hole
  box {<-1,3,-1>,<3,4,2>}  // top gap
  translate < -2.5, -2.5, 0 >
}


// letter S
#declare LetterS = difference {
  union {
    box {<1,4,0>,<4,5,1>}  // top horizontal bar
    box {<1,2,0>,<4,3,1>}  // middle horizontal bar
    box {<1,0,0>,<4,1,1>}  // bottom horizontal bar

    box {<4,1,0>,<5,2,1>}  // bottom right vertical bar
    box {<4,3.5,0>,<5,4,1>}  // top right serif
    box {<0,1,0>,<1,1.5,1>}  // bottom left serif

    cylinder {<1,4,0>,<1,4,1>,1}  // top left
    cylinder {<4,4,0>,<4,4,1>,1}  // top right
    cylinder {<1,3,0>,<1,3,1>,1}  // middle left
    cylinder {<4,2,0>,<4,2,1>,1}  // middle right
    cylinder {<1,1,0>,<1,1,1>,1}  // bottom left
    cylinder {<4,1,0>,<4,1,1>,1}  // bottom right
  }
  box {<-1,3,-1>,<6,4,2>}  // top horizontal gap
  box {<-11,1,-1>,<4,2,2>}  // bottom horizontal gap

  //box {<3,3,-1>,<6,3.5,2>}  // top serif gap
  //box {<-1,1.5,-1>,<2,2,2>}  // bottom serif gap

  translate < -2.5, -2.5, 0 >
}


// word PROCESSOR
#declare WordPROCESSOR = union {
  object { LetterP
    translate -24*x
  }
  object { LetterR
    translate -18*x
  }
  object { LetterO
    translate -12*x
  }
  object { LetterC
    translate -6*x
  }
  object { LetterE
    //translate 0*x
  }
  object { LetterS
    translate 6*x
  }
  object { LetterS
    translate 12*x
  }
  object { LetterO
    translate 18*x
  }
  object { LetterR
    translate 24*x
  }
}


// letter T
#declare LetterT = union {
  box { < 2, 0, 0 >, < 3, 5, 1 > }  // vertical bar
  box { < 0, 4, 0 >, < 5, 5, 1 > }  // horizontal bar
  translate < -2.5, -2.5, 0 >
}

// letter H
#declare LetterH = union {
  box { < 0, 0, 0 >, < 1, 5, 1 > }  // left vertical bar
  box { < 4, 0, 0 >, < 5, 5, 1 > }  // right vertical bar
  box { < 0, 2, 0 >, < 5, 3, 1 > }  // horizontal bar
  translate < -2.5, -2.5, 0 >
}

// Letter A
#declare normA1 = vnormalize( < 5, 2, 0 > );  // right side slope
#declare LetterA = union {
  box { < 1, 1, 0 >, < 4, 2, 1 > }  // horizontal bar
  intersection {  // outer character limits
    union {
      intersection {  // right slope
        plane { normA1, 0   // outer edge
          translate 5*x
        }
        plane { -normA1, 0   // inner edge
          translate -normA1 + 5*x
        }
        plane { -x, 0  // prevent peeking out upper side
          translate 2*x
        }
      }
      intersection {  // left slope
        plane { normA1, 0   // outer edge
          translate 5*x
        }
        plane { -normA1, 0   // inner edge
          translate -normA1 + 5*x
        }
        plane { -x, 0  // prevent peeking out upper side
          translate 2*x
        }
	translate -2.5*x  // mirror right slope around letter's center
	scale < -1, 1, 1 >
	translate 2.5*x
      }
    }
    
    box { < 0, 0, 0 >, < 5, 5, 1 > }
  }
  translate < -2.5, -2.5, 0 >
}

// apostrophe
#declare InterpunctionApostrophe = box { <2,3,0>,<3,5,1>
  translate < -2.5, -2.5, 0 >
}

// word THAT'S
#declare WordTHATS = union {
  object { LetterT
    translate -13*x
  }
  object { LetterH
    translate -7*x
  }
  object { LetterA
    translate -1*x
  }
  object { LetterT
    translate 4*x
  }
  object { InterpunctionApostrophe
    translate 8*x
  }
  object { LetterS
    translate 12*x
  }


  //translate 0.5*x
  //scale 1/5.5
}

// words THAT'S A NICE
#declare WordTHATSANiCE = union {
  object { WordTHATS
    translate -18*x
  }
  object { LetterA
    translate 4*x
  }
  object { WordNiCE
    translate 21*x
  }

  translate 1.5*x
  //scale 1/13
}


// letter Y
#declare normY1 = vnormalize( < 3, -2, 0 > );  // right side slope
#declare LetterY = union {
  box { < 2, 0, 0 >, < 3, 2, 1 > }  // middle vertical bar
  intersection {  // outer limits of sloped parts
    union {
      intersection {  // right slope
        plane { normY1, 0
          translate < 3, 2, 0 >
        }
        plane { -normY1, 0
          translate -normY1
          translate < 3, 2, 0 >
        }
        plane { -x, 0  // prevent peeking out the middle
          translate 2*x
        }
      }
      
      intersection {  // left slope
        plane { normY1, 0
          translate < 3, 2, 0 >
        }
        plane { -normY1, 0
          translate -normY1
          translate < 3, 2, 0 >
        }
        plane { -x, 0  // prevent peeking out the middle
          translate 2*x
        }
	translate -2.5*x  // mirror right slope around letter's center
	scale < -1, 1, 1 >
	translate 2.5*x
      }
    }
    box { < 0, 2, 0 >, < 5, 5, 1 > }
  }
  translate < -2.5, -2.5, 0 >
}

// letter U
#declare LetterU = difference {
  union {  // solid parts
    box { < 0, 1, 0 >, < 5, 5, 1 > }  // left to right
    box { < 1, 0, 0 >, < 4, 5, 1 > }  // bottom centered
    cylinder {<1,1,0>,<1,1,1>,1}  // bottom left
    cylinder {<4,1,0>,<4,1,1>,1}  // bottom right
  }
  union {  // empty parts
    box { < 1, 1, -1 >, < 4, 6, 2 > }
  }
  translate < -2.5, -2.5, 0 >
}

// word YOU
#declare WordYOU = union {
  object { LetterY
    translate -6*x
  }
  object { LetterO
    translate 0*x
  }
  object { LetterU
    translate 6*x
  }
}

// letter G
#declare LetterG = union {
  object { LetterC
  translate < 2.5, 2.5, 0 >
    translate -3*z
  }
  box { < 4, 1, 0 >, < 5, 3, 1 > }  // right vertical bar
  box { < 3, 2, 0>, < 5, 3, 1 > }  // middle horizontal bar

  translate < -2.5, -2.5, 0 >
}

// word GOT
#declare WordGOT = union {
  object { LetterG
    translate -6*x
  }
  object { LetterO
    translate 0*x
  }
  object { LetterT
    translate 6*x
  }
}

// word THERE
#declare WordTHERE = union {
  object { LetterT
    translate -12*x
  }
  object { LetterH
    translate -6*x
  }
  object { LetterE
    translate 0*x
  }
  object { LetterR
    translate 6*x
  }
  object { LetterE
    translate 12*x
  }
}

// interpunction exclamation mark
#declare InterpunctionExclamation = union {
  box { < 2, 0, 0 >, < 3, 1, 1 > }  // dot
  box { < 2, 1.5, 0 >, < 3, 5, 1 > }  // vertical bar
  translate < -2.5, -2.5, 0 >
}


// words YOU GOT THERE!
#declare WordYOUGOTTHERE = union {
  object { WordYOU
    translate -28*x
  }
  object { WordGOT
    translate -7*x
  }
  object { WordTHERE
    translate 20*x
  }
  object { InterpunctionExclamation
    translate 36*x
  }
}


union {  // all three lines of text
  object { WordTHATSANiCE
    translate 8*y
  }
  object { WordPROCESSOR }
  object { WordYOUGOTTHERE
    translate -8*y
  }
  scale 1/14
}

background { color rgb 0.2 }
